------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops (TSM3 Module) 
------------------------------------------------------------------------------
-- Modules/GUI.lua - data display
--
-- Author: Caraxe
--
-- Version v0.3.3
------------------------------------------------------------------------------

local addonName, addon = ...
local GUI = addon:NewModule("GUI", "AceEvent-3.0", "AceHook-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}

GUIDEBUG = nil

GUI.frScaleValue = 0.8
GUI.rarityList = {}
for i = 0, 8 do
	GUI.rarityList[i] = ITEM_QUALITY_COLORS[i].hex .. 
		_G[format("ITEM_QUALITY%d_DESC", i)] .. 
		FONT_COLOR_CODE_CLOSE
end

GUI.timeFormat = {}
GUI.timeFormat["ago"]  = L["_ Hr. _ Min. ago"]
GUI.timeFormat["%d.%m.%Y %H:%M:%S"] = "DD.MM.YYYY HH:MM"
GUI.timeFormat["%Y-%m-%d %H:%M:%S"] = "YYYY-MM-DD HH:MM"

GUI.VIEW_SESSION = 1
GUI.VIEW_DATABASE = 2
GUI.VIEW_CHALLENGE = 3
GUI.VIEW_INV = 4
GUI.VIEW_TOOLS = 5
GUI.VIEW_INFO = 6
GUI.view = GUI.VIEW_SESSION

GUI.isDebug = false

function GUI:ToggleDebug()
	addon.GUI.isDebug = not addon.GUI.isDebug
	if (not addon.GUI.isDebug) then
		addon:Printf(L["Extended GUI off"])
	else
		addon.Data:DebugPrintf(L["Extended GUI on"])
	end
end

------------------------------------------------------------------------------
-- Show addon settings
--
function GUI:ShowInfo()
	if ((addon.db.realm.sesTime + 3 >= time()) and addon.db.realm.sesEnd == 0) then
		addon:Printf(L["Starting a new session."]);
	elseif (addon.db.realm.sesEnd == 0) then
		addon:Printf(L["Continuing a previous session. Current Looted Item Value is %s"], GetCoinTextureString(addon.db.realm.sesMV));
	else
		addon:Print(L["Session is stopped. A new session starts after saving or wiping the current session data."])
	end
	addon:Printf(L["  Quality filter is >=%s."], GUI.rarityList[addon.db.global.minRarity])
	addon:Printf(L["  Noteworthy items are worth >%s."], addon.db.global.minNoteworthy);
	-- not important 
	--[[
	if (addon.db.global.useVendorSellForBOP and addon.db.global.useVendorSellForGrays) then
		addon:Printf(L["  VendorSell prices are used for BOPs and Grays."])
	elseif (addon.db.global.useVendorSellForBOP) then
		addon:Printf(L["  VendorSell prices are used for BOPs and items without Market Value"])
	elseif (addon.db.global.useVendorSellForGrays) then
		addon:Printf(L["  VendorSell prices are used for Grays."])
	end
	if (addon.db.global.useNotTheRealItemString) then
		addon:Printf(L["  Item variants and upgrades are discarded for compatibility reasons."])
		addon:Printf(L["  VendorSell prices may be wrong."])
	else
		addon:Printf(L["  VensorSell prices are used for item variants and upgrades."])
	en
	--]]
	if (addon.db.global.mvSource == "Custom") then
		addon:Printf(L["  A custom price source '%s' is used."], addon.db.global.mvCustomSource)
	else
		mvSourcesList = TSMAPI:GetPriceSources() or {}
		addon:Printf(L["  Price source is '%s'."], 
			mvSourcesList[addon.db.global.mvSource] or addon.db.global.mvSource)
	end
	addon:Print(L["Good luck farming items!"])
end

------------------------------------------------------------------------------
-- Container
--
function GUI:GUIDebugFrames(parent, container, group)
	addon.Data:DebugPrintf("... p=%s, c=%s, t=%s", 
		tostring(parent), tostring(container), tostring(group))
end

function GUI:Load(parent)
	addon.Data:DebugPrintf("GUI:Load()")
	addon.Data:IsBlacklisted("")
	
	addon:CheckSettings()

	-- make the windows smaller during farming
	if TSMMainFrame1 then
		if (addon.db.global.frScale) then
			TSMMainFrame1.SetFrameScale(TSMMainFrame1, GUI.frScaleValue)
		end
	end

	-- build the tab group
	local container = AceGUI:Create("SimpleGroup")
	container:SetLayout("Fill")
	parent:AddChild(container)
	GUI.isVisible = true

	local tabGroup = AceGUI:Create("TSMTabGroup")
	
	GUI.fmSesParent = parent
	GUI.fmSesContainer = container
	GUI.fmSesGroup = tabGroup
	GUI:GUIDebugFrames(GUI.fmSesParent, GUI.fmSesContainer, GUI.fmSesGroup)
	
	tabGroup:SetLayout("Fill")
	local tab1 = {text = L["Session"], value = GUI.VIEW_SESSION}
	local tab2 = {text = L["History"], value = GUI.VIEW_DATABASE}
	local tab3 = {text = L["Challenges"], value = GUI.VIEW_CHALLENGE}
	local tab4 = {text = L["Inventory"], value = GUI.VIEW_INV}
	local tab5 = {text = L["Tools"], value = GUI.VIEW_TOOLS}
	local tab6 = {text = L["Information"], value = GUI.VIEW_INFO}
	if (GUI.isDebug) then
		tabGroup:SetTabs({
			tab1, 
			tab2, 
			-- tab3, 
			tab4, 
			-- tab5, 
			tab6,
			});
	else
		tabGroup:SetTabs({
			tab1, 
			tab2, 
			-- tab3, 
			-- tab4, 
			-- tab5, 
			tab6,
			});
	end
	tabGroup:SetCallback("OnGroupSelected", function(self, _, value)
		tabGroup:ReleaseChildren()
		if value == GUI.VIEW_SESSION then
			GUI:LoadSession(self)
		elseif value == GUI.VIEW_DATABASE then
			GUI:LoadDatabase(self)
		elseif value == GUI.VIEW_CHALLENGE then
			GUI:LoadChallenge(self)
		elseif value == GUI.VIEW_INV then
			GUI:LoadInventory(self)
		elseif value == GUI.VIEW_TOOLS then
			GUI:LoadTools(self)
		elseif value == GUI.VIEW_INFO then
			GUI:LoadInfo(self)
		end
		tabGroup.children[1]:DoLayout()
	end)
	container:AddChild(tabGroup)

	tabGroup:SelectTab(GUI.VIEW_SESSION)

	GUI:HookScript(container.frame, "OnHide", function()
		addon.Data:DebugPrintf("OnHide(%s)", tostring(container.frame))
		GUI:UnhookAll()
		GUI.isVisible = false
	end)
end

------------------------------------------------------------------------------
-- Session Tab
--
function GUI:LoadSession(container)
	addon.Data:DebugPrintf("GUI:LoadSession()")
	GUI.view = GUI.VIEW_SESSION

	-- Filter: time period
	local timeList = {[99]=L["All"], [0]=L["Today"], [1]=L["Yesterday"]}
	for _, days in ipairs({7, 14, 21, 28}) do
		timeList[days] = format(L["Last %d Days"], days)
	end

	-- Filter: only given player
	local playerList = {}
	local zoneList = {}
	for _, data in pairs(addon.sesItems) do
		for _, record in ipairs(data.drops) do
			playerList[record.player] = record.player
			zoneList[record.zone] = record.zone
		end
	end
	playerList["all"] = L["All"]
	zoneList["all"] = L["All"]
	sort(playerList, function(a, b) return (a.player or 0) < (b.player or 0) end)
	sort(zoneList, function(a, b) return (a.zone or 0) < (b.zone or 0) end)
	
	-- Filter: default filter is empty
	GUI.sesFilters = {}

	-- compatibilty warning
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	local nextLapPopupText = L["Click this button to start a new lap."]
	if (addon.db.global.lpInstanceReset) then
		nextLapPopupText = L["Click this button to start a new lap and reset all instances."]
	end
	
	local stHandlers = {
		OnEnter = function(_, data, self)
			if not data then return end
			if not data.itemString then
				addon.Data:DebugPrintf("Tooltip: data.itemString is nil!?")
			else
				-- Show the real item tooltip in case of a shortened itemString or the item tooltip of a
				-- shortened item which is not in my bags?
				-- In case of an upgrade I would like to see the real item version, not the shortend/base item:
				-- I want to see the stats, so default is case 2 below!

				-- case 1: show the base link
				--local lItemString = TSMAPI.Item:ToItemString(data.itemString)
				--if (addon.db.global.useNotTheRealItemString) then
				--	lItemString = TSMAPI.Item:ToBaseItemString(lItemString)
				--end
				--addon.Data:DebugPrintf("Tooltip for %s: %s, Real %s", lItemString, link, data.itemString)
				-- local link = TSMAPI.Item:ToItemLink(lItemString)
				
				-- case 2: show the real link
				local link = TSMAPI.Item:ToItemLink(data.itemString)
				
				GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
				TSMAPI.Util:SafeTooltipLink(link)
			end
		end,
		OnLeave = function()
			GameTooltip:Hide()
		end,
	}

	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "InlineGroup",
					title = L["Session Control"],
					layout = "Flow",
					children = {
						-- lblContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Label",
									text = L["Session Start"],
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["Duration / Lap #"],
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["LIV / Gold"],
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["Price Source"], 
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = ITEM_QUALITY_COLORS[addon.db.global.minRarity].hex .. L["Items"] .. FONT_COLOR_CODE_CLOSE ..
									" / " .. ">" .. addon.db.global.minNoteworthy,
									relativeWidth = 0.2,
								},
							},
						},	
						-- infoContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "InteractiveLabel",
									text = L["LIV"],
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.2,
								},
							},
						},	
						-- btnContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Button",
									text = L["Stop"],
									relativeWidth = 0.19,
									callback = function() 
										if (addon.db.realm.sesEnd == 0) then
											-- Stop Session
											addon.db.realm.sesEnd = time()
											addon:Printf(L["Stopping the session after %s."],
												SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime))
										else
											addon:Printf(L["Session was already stopped %s ago"], SecondsToTime(time() - addon.db.realm.sesEnd))
										end
										end,
									tooltip = L["Click this button to stop a session."],
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Button",
									text = L["Next"],
									relativeWidth = 0.19,
									callback = function() 
										if (addon.db.realm.sesEnd == 0) then
											addon.db.realm.lapTime = time()
											addon.db.realm.lapNum = addon.db.realm.lapNum + 1
											GUI.sesFilters = {}
											-- private.GetSessionInfoData(GUI.sesFilters)
											-- TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
											if (addon.db.global.nwSound ~= "") then
												PlaySound(addon.db.global.nwSound, "master")
											end
											if (addon.db.global.lpInstanceReset) then
												ResetInstances()
												addon:Printf(L["Resetting the instance."])
											end
											addon:Printf(L["Finishing Lap #%s with Looted Item Value %s."],
												addon.db.realm.lapNum - 1, GetCoinTextureString(addon.db.realm.sesMV))
											container:Reload()
										else
											addon:Printf(L["Can't start lap of a stopped session."])
										end
									end,
									tooltip = nextLapPopupText,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "InteractiveLabel",
									text = L["Gold"],
									relativeWidth = 0.19,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Button",
									text = L["Save"],
									relativeWidth = 0.19,
									callback = function() 
										if (addon.db.global.nwSound ~= "") then
											PlaySound(addon.db.global.nwSound, "master")
											addon.Data:DebugPrintf("Play %s", addon.db.global.nwSound)  
										end
										addon.Data:SessionToDatabase()
										GUI.sesFilters = {}
										addon.Data:WipeSession()
										GUI:ShowInfo()
										addon:UpdateBroker()
										addon.Data:ResetCurrentTag(true)
										container:Reload()
										end,
									tooltip = L["Click this button to save the current session to the database and start a new session."],
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Button",
									text = L["Wipe"],
									relativeWidth = 0.2,
									callback = function() 
										if (addon.db.global.nwSound ~= "") then
											PlaySound(addon.db.global.nwSound, "master")
											addon.Data:DebugPrintf("Play %s", addon.db.global.nwSound)  
										end
										GUI.sesFilters = {}
										addon.Data:WipeSession()
										GUI:ShowInfo()
										addon:UpdateBroker()
										container:Reload()
									end,
									tooltip = L["Click this button to wipe the current session data and start a new session."],
								},
							},
						},
					},
				},
				{
					type = "InlineGroup",
					title = L["Session Data"],
					layout = "TSMFillList",
					children = {
						{
							type = "SimpleGroup",
							layout = "Flow",
							children = {
								{
									type = "EditBox",
									label = L["Search"],
									relativeWidth = 0.16,
									onTextChanged = true,
									callback = function(_, _, value)
										value = value:trim()
										if value == "" then
											GUI.sesFilters.name = nil
										else
											GUI.sesFilters.name = TSMAPI.Util:StrEscape(value)
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
								{
									type = "GroupBox",
									label = L["Group"],
									relativeWidth = 0.16,
									callback = function(_, _, value)
										GUI.sesFilters.group = value
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Rarity"],
									relativeWidth = 0.16,
									list = GUI.rarityList,
									value = 0,
									callback = function(_, _, key)
										if key > 0 then
											GUI.sesFilters.rarity = key
										else
											GUI.sesFilters.rarity = nil
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Player"],
									relativeWidth = 0.16,
									list = playerList,
									value = "all",
									callback = function(_, _, value)
										if value == "all" then
											GUI.sesFilters.player = nil
										else
											GUI.sesFilters.player = value
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Timeframe"],
									relativeWidth = 0.16,
									list = timeList,
									value = GUI.sesFilters.time or 99,
									callback = function(_, _, value)
										if value == 99 then
											GUI.sesFilters.time = nil
										else
											GUI.sesFilters.time = value
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Zone"],
									relativeWidth = 0.20,
									list = zoneList,
									value = "all",
									callback = function(_, _, value)
										if value == "all" then
											GUI.sesFilters.zone = nil
										else
											GUI.sesFilters.zone = value
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
										private.GetSessionInfoData(GUI.sesFilters)
									end,
								},
							},
						},
						{ 
							type = "ScrollingTable",
							tag = "SID_SESSION_ST",
							relativeWidth = 1,
							colInfo = {
								{name=L["Item"] .. flNotTheRealItemString, width=0.30, headAlign="LEFT"},
								{name=L["Item Value"], width=0.17, align="RIGHT", headAlign="LEFT"},
								{name=L["Qty"], width=0.05, align="RIGHT", headAlign="LEFT"},
								{name=L["Player"], width=0.12, headAlign="LEFT"},
								{name=L["Zone"], width=0.20, headAlign="LEFT"},
								{name=L["Time"], width=0.16, headAlign="LEFT"},
							},
							handlers = stHandlers,
							defaultSort = -6,
							selectionDisabled = true,
						},
					},
				},
			},
		},
	}

	local stInfo = page[1].children[2].children[2]
	if not addon.db.global.shPlayer then
		stInfo.colInfo = {
			{name=L["Item"] .. flNotTheRealItemString, width=0.40, headAlign="LEFT"},
			{name=L["Item Value"], width=0.16, align="RIGHT", headAlign="LEFT"},
			{name=L["Qty"], width=0.04, align="RIGHT", headAlign="LEFT"},
			{name="", width=0.01, headAlign="LEFT"},
			{name=L["Zone"], width=0.19, headAlign="LEFT"},
			{name=L["Time"], width=0.20, headAlign="LEFT"},
		}
	end
	
	TSMAPI.GUI:BuildOptions(container, page)

	-- yes, we can change lable texts
	GUI.lblSesContainer = container.children[1].children[1].children[1]
	GUI.infoSesContainer = container.children[1].children[1].children[2]
	GUI.btnSesContainer = container.children[1].children[1].children[3]
	
	-- scale LIV / Gold display a little bit
	local font, size = GUI.infoSesContainer.children[5].label:GetFont()
	GUI.infoSesContainer.children[5].label:SetJustifyH("RIGHT")
	GUI.infoSesContainer.children[5].label:SetFont(font, 20)
	GUI.btnSesContainer.children[5].label:SetJustifyH("RIGHT")
	GUI.btnSesContainer.children[5].label:SetFont(font, 20)
	
	TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
	private.GetSessionInfoData(GUI.sesFilters)
	addon:UpdateBroker()
end

function GUI:UpdateSessionData(sesItemsUpdated)
	addon.Data:DebugPrintf("GUI:UpdateSessionData()")
	-- GUI:GUIDebugFrames(GUI.fmSesParent, GUI.fmSesContainer, GUI.fmSesGroup)
	if GUI.view == GUI.VIEW_SESSION and GUI.isVisible then
		if (sesItemsUpdated) then
			TSMAPI.GUI:UpdateTSMScrollingTableData("SID_SESSION_ST", private.GetSessionSTItemData(GUI.sesFilters))
		end
		private.GetSessionInfoData(GUI.sesFilters)
	end
	addon:UpdateBroker()
end

function private.GetSessionSTItemData(lfilters)
	addon.Data:DebugPrintf("GetSessionSTItemData()")
	if GUI.view ~= GUI.VIEW_SESSION or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end

	return private.GetSTItemData(addon.sesItems, lfilters)
end

function private.GetSTItemData(items, lfilters, showBOP)
	local stData = {}
	local filters = lfilters or {}

	for itemString, data in pairs(items) do
		if #data.drops > 0 and not private:IsItemFiltered(itemString, filters) then
			for _, record in ipairs(data.drops) do
				if not private:IsRecordFiltered(record, filters) then
					local isVariant = data.iString ~= data.iBaseString
					local lItem = ""
					local marketValue = 0
					if (addon.db.global.useNotTheRealItemString) then
						lItem = TSMAPI.Item:ToItemLink(data.iBaseString)
						if (isVariant) then
							lItem = lItem .. ITEM_QUALITY_COLORS[5].hex .. " !" .. FONT_COLOR_CODE_CLOSE
						end
						marketValue = addon.Data:GetItemValue(data.iBaseString)
					else
						lItem = itemString
						marketValue = addon.Data:GetItemValue(data.iString)
					end

					--addon.Data:DebugPrintf("Data for %s, Real %s (%s), Base (%s) %s, Used %s", 
					--	itemString, 
					--	lItemString, GetCoinTextureString(addon.Data:GetItemValue(lItemString)), 
					--	lBaseItemString, GetCoinTextureString(addon.Data:GetItemValue(lBaseItemString)),
					--	lItem)
					
					if (showBOP) then
						if not (addon.isInfight) then
							if (TSMAPI.Item:IsSoulbound(itemString)) then
								lItem = lItem .. ITEM_QUALITY_COLORS[8].hex .. " BOP" .. FONT_COLOR_CODE_CLOSE
							end
						end
					end
					
					if addon.db.global.shPlayer then
						local row = {
							cols = {
								{
									value = lItem,
									sortArg = data.iName,
								},
								{
									value = (GUI:MoneyToString(marketValue) or "|cff999999---|r") .. "  ",
									sortArg = marketValue or 0,
								},
								{
									value = record.quantity .. "  ",
									sortArg = record.quantity,
								},
								{
									value = record.player,
									sortArg = record.player,
								},
								{
									value = record.zone,
									sortArg = record.zone,
								},
								{
									value = GUI:TimeToString(record.time),
									sortArg = record.time,
								},
							},
							itemString = itemString,
						}
						tinsert(stData, row)
					else
						local row = {
							cols = {
								{
									value = lItem,
									sortArg = data.iName,
								},
								{
									value = (GUI:MoneyToString(marketValue) or "|cff999999---|r") .. "  ",
									sortArg = marketValue or 0,
								},
								{
									value = record.quantity .. "  ",
									sortArg = record.quantity,
								},
								{
									value = "",
									sortArg = record.player,
								},
								{
									value = record.zone,
									sortArg = record.zone,
								},
								{
									value = date(addon.timeFormat,record.time),
									sortArg = record.time,
								},
							},
							itemString = itemString,
						}
						tinsert(stData, row)
					end
				end
			end
		end
	end
	return stData
end

function private.GetSessionInfoData(filters)
	addon.Data:DebugPrintf("GetSessionInfoData()")
	if GUI.view ~= GUI.VIEW_SESSION or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end

	local stData = {}
	local mvFilter = 0
	local nwFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local isFiltered = false

	if (not filters.name and not filters.group and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		addon.Data:DebugPrintf("... with no filters")
		-- FIXME: check if price source was changed for sum up sesMV
	else
		addon.Data:DebugPrintf("... with filters")
		isFiltered = true
		for itemString, data in pairs(addon.sesItems) do
			if #data.drops > 0 then
				itsSum = itsSum + 1
			end
			if #data.drops > 0 and not private:IsItemFiltered(itemString, filters) then
				itsFilter = itsFilter + 1
				for _, record in ipairs(data.drops) do
					itSum = itSum + record.quantity
					if not private:IsRecordFiltered(record, filters) then
						itFilter = itFilter + record.quantity
						local lItemString = data.iString
						if (addon.db.global.useNotTheRealItemString) then
							lItemString = data.iBaseString
						end
						local marketValue = addon.Data:GetItemValue(lItemString) or 0
						mvFilter = mvFilter + (marketValue * record.quantity)
						if TSMAPI:MoneyFromString(addon.db.global.minNoteworthy) then
							if (marketValue > TSMAPI:MoneyFromString(addon.db.global.minNoteworthy)) then
								nwFilter = nwFilter + record.quantity
							end
						end
					end
				end
			end
		end
		--addon.Data:DebugPrintf("  itsSum %s itsFilter %s itSum %s itFilter %s mvFilter %s nwFilter %s",
		--	itsSum, itsFilter, itSum, itFilter, mvFilter, nwFilter)
		--addon.Data:DebugPrintf("  sesMV %s sesGold %s sesNumItems %s sesNumNW %s",
		--	addon.db.realm.sesMV, addon.db.realm.sesGold, addon.db.realm.sesNumItems, addon.db.realm.sesNumNW)
	end
	
	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if ((GUI.infoSesContainer) and (GUI.btnSesContainer)) then
			-- Session Start Time
			GUI.lblSesContainer.children[1]:SetText(L["Data Source"])
			GUI.infoSesContainer.children[1]:SetText(ITEM_QUALITY_COLORS[5].hex .. 
				L["Selection"] 
				.. FONT_COLOR_CODE_CLOSE)
			-- Duration / Lap #
			GUI.lblSesContainer.children[3]:SetText("")
			GUI.infoSesContainer.children[3]:SetText("")
			-- LIV / Gold
			GUI.lblSesContainer.children[5]:SetText(L["LIV"])
			GUI.infoSesContainer.children[5]:SetText(GUI:MoneyToString(mvFilter)) 
			GUI.infoSesContainer.children[5].label:SetAllPoints()
			GUI.btnSesContainer.children[5]:SetText("") 
			-- GUI.btnSesContainer.children[5].label:SetAllPoints()
			-- Price Source
			GUI.infoSesContainer.children[7]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
			-- Items / Noteworthy
			GUI.infoSesContainer.children[9]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				itFilter .. " / " .. nwFilter 
				..  FONT_COLOR_CODE_CLOSE)
		end
	else 
		if ((GUI.infoSesContainer) and (GUI.btnSesContainer)) then
			-- Session Start Time
			GUI.lblSesContainer.children[1]:SetText(L["Session Start"])
			if (addon.db.global.timeFormat == "ago") then
				if (GetLocale() == "deDE") then
					GUI.infoSesContainer.children[1]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
						date(addon.timeFormat, addon.db.realm.sesTime) 
						.. FONT_COLOR_CODE_CLOSE)
				else
					GUI.infoSesContainer.children[1]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
						date(addon.timeFormat, addon.db.realm.sesTime) 
						.. FONT_COLOR_CODE_CLOSE)
				end
			else
				GUI.infoSesContainer.children[1]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
					date(addon.timeFormat, addon.db.realm.sesTime) 
					.. FONT_COLOR_CODE_CLOSE)
			end
			-- Duration / Lap #
			if (addon.db.realm.sesEnd == 0) then
				GUI.lblSesContainer.children[3]:SetText(L["Duration / Lap #"])
				GUI.infoSesContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
					SecondsToTime(time() - addon.db.realm.sesTime) .. " / " .. addon.db.realm.lapNum 
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.lblSesContainer.children[3]:SetText(L["Duration"])
				GUI.infoSesContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				SecondsToTime(addon.db.realm.sesEnd - addon.db.realm.sesTime) 
				.. FONT_COLOR_CODE_CLOSE)
			end
			-- Looted Item Value / Gold
			GUI.lblSesContainer.children[5]:SetText(L["LIV / Gold"])
			GUI.infoSesContainer.children[5]:SetText(GUI:MoneyToString(addon.db.realm.sesMV))
			GUI.infoSesContainer.children[5].label:SetAllPoints()
			GUI.btnSesContainer.children[5]:SetText(GUI:MoneyToString(addon.db.realm.sesGold))
			GUI.btnSesContainer.children[5].label:SetAllPoints()
			-- Price Source
			GUI.infoSesContainer.children[7]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
			-- Items / Noteworthy
			GUI.infoSesContainer.children[9]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.realm.sesNumItems .. " / " .. addon.db.realm.sesNumNW 
				..  FONT_COLOR_CODE_CLOSE)
		end
	end
end

------------------------------------------------------------------------------
-- Database Tab
--
function GUI:LoadDatabase(container)
	addon.Data:DebugPrintf("GUI:LoadDatabase()")
	GUI.view = GUI.VIEW_DATABASE
	
	-- Load History Data
	addon.Data:LoadDatabase()

	local tagList = {}
	for tag in pairs(addon.hisItems) do
		tagList[tag] = date(addon.timeFormat, tag)
		if (addon.hisInfo[tag].hisZone) then
			tagList[tag] = tagList[tag] .. " / " .. addon.hisInfo[tag].hisZone
		end
	end

	local zoneList = {}
	if (addon.hisCTag > 0) then
		for _, data in pairs(addon.hisItems[addon.hisCTag]) do
			for _, record in ipairs(data.drops) do
				zoneList[record.zone] = record.zone
			end
		end
	end
	zoneList["all"] = L["All"]
	sort(zoneList, function(a, b) return (a.zone or 0) < (b.zone or 0) end)
	
	-- Filter: default filter is empty
	GUI.hisFilters = {}

	-- compatibilty warning
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	local stHandlers = {
		OnEnter = function(_, data, self)
			if not data then return end
			if not data.itemString then
				addon.Data:DebugPrintf("Tooltip: data.itemString is nil!?")
			else
				-- Show the real item tooltip in case of a shortened itemString or the item tooltip of a
				-- shortened item which is not in my bags?
				-- In case of an upgrade I would like to see the real item version, not the shortend/base item:
				-- I want to see the stats, so default is case 2 below!

				-- case 1: show the base link
				--local lItemString = TSMAPI.Item:ToItemString(data.itemString)
				--if (addon.db.global.useNotTheRealItemString) then
				--	lItemString = TSMAPI.Item:ToBaseItemString(lItemString)
				--end
				--addon.Data:DebugPrintf("Tooltip for %s: %s, Real %s", lItemString, link, data.itemString)
				-- local link = TSMAPI.Item:ToItemLink(lItemString)
				
				-- case 2: show the real link
				local link = 
				TSMAPI.Item:ToItemLink(data.itemString)
				
				GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
				TSMAPI.Util:SafeTooltipLink(link)
			end
		end,
		OnLeave = function()
			GameTooltip:Hide()
		end,
	}

	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "InlineGroup",
					title = L["Session History Selection"],
					layout = "Flow",
					children = {
						-- lblContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Label",
									text = L["Session"],
									relativeWidth = 0.45,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["Duration"],
									relativeWidth = 0.14,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["LIV"],
									relativeWidth = 0.20,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["Price Source"], 
									relativeWidth = 0.18,
								},
							},
						},	
						-- infoContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Dropdown",
									relativeWidth = 0.45,
									list = tagList,
									value = addon.hisCTag,
									callback = function(_, _, value)
										addon.Data:DebugPrintf(L["History Selection, value=%s"], value)
										addon.hisCTag = value
										addon.Data:UpdateDatabaseTag(addon.hisCTag)
										container:Reload()
										--TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
										--private.GetHistoryInfoData(GUI.hisFilters)
									end,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.14,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = L["LIV"],
									relativeWidth = 0.20,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.01,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.18,
								},
							},
						},	
					},
				},
				{
					type = "InlineGroup",
					title = L["Session History Data"],
					layout = "TSMFillList",
					children = {
						{
							type = "SimpleGroup",
							layout = "Flow",
							children = {
								{
									type = "EditBox",
									label = L["Search"],
									relativeWidth = 0.20,
									onTextChanged = true,
									callback = function(_, _, value)
										value = value:trim()
										if value == "" then
											GUI.hisFilters.name = nil
										else
											GUI.hisFilters.name = TSMAPI.Util:StrEscape(value)
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
										private.GetHistoryInfoData(GUI.hisFilters)
									end,
								},
								{
									type = "GroupBox",
									label = L["Group"],
									relativeWidth = 0.20,
									callback = function(_, _, value)
										GUI.hisFilters.group = value
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
										private.GetHistoryInfoData(GUI.hisFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Rarity"],
									relativeWidth = 0.20,
									list = GUI.rarityList,
									value = 0,
									callback = function(_, _, key)
										if key > 0 then
											GUI.hisFilters.rarity = key
										else
											GUI.hisFilters.rarity = nil
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
										private.GetHistoryInfoData(GUI.hisFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Zone"],
									relativeWidth = 0.20,
									list = zoneList,
									value = "all",
									callback = function(_, _, value)
										if value == "all" then
											GUI.hisFilters.zone = nil
										else
											GUI.hisFilters.zone = value
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
										private.GetHistoryInfoData(GUI.hisFilters)
									end,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.10,
								},
								{
									type = "Button",
									text = L["Delete"],
									relativeWidth = 0.10,
									callback = function() 
										if (addon.db.global.nwSound ~= "") then
											PlaySound(addon.db.global.nwSound, "master")
											addon.Data:DebugPrintf("Play %s", addon.db.global.nwSound)  
										end
										addon.Data:WipeHistoryTag(addon.hisCTag)
										addon.hisCTag = 0
										addon.Data:ResetCurrentTag(true)
										addon.Data:UpdateDatabaseTag(addon.hisCTag)
										container:Reload()
										end,
									tooltip = L["Click this button to delete the current session history data."],
								},
							},
						},
						{ 
							type = "ScrollingTable",
							tag = "SID_HISTORY_ST",
							relativeWidth = 1,
							colInfo = {
								{name=L["Item"] .. flNotTheRealItemString, width=0.30, headAlign="LEFT"},
								{name=L["Item Value"], width=0.17, align="RIGHT", headAlign="LEFT"},
								{name=L["Qty"], width=0.05, align="RIGHT", headAlign="LEFT"},
								{name=L["Player"], width=0.12, headAlign="LEFT"},
								{name=L["Zone"], width=0.20, headAlign="LEFT"},
								{name=L["Time"], width=0.16, headAlign="LEFT"},
							},
							handlers = stHandlers,
							defaultSort = -6,
							selectionDisabled = true,
						},
					},
				},
			},
		},
	}

	local stInfo = page[1].children[2].children[2]
	if not addon.db.global.shPlayer then
		stInfo.colInfo = {
			{name=L["Item"] .. flNotTheRealItemString, width=0.40, headAlign="LEFT"},
			{name=L["Item Value"], width=0.16, align="RIGHT", headAlign="LEFT"},
			{name=L["Qty"], width=0.04, align="RIGHT", headAlign="LEFT"},
			{name="", width=0.01, headAlign="LEFT"},
			{name=L["Zone"], width=0.19, headAlign="LEFT"},
			{name=L["Time"], width=0.20, headAlign="LEFT"},
		}
	end
	
	TSMAPI.GUI:BuildOptions(container, page)
	
		-- yes, we can change label texts
	GUI.lblHisContainer = container.children[1].children[1].children[1]
	GUI.infoHisContainer = container.children[1].children[1].children[2]
	
	-- scale LIV / Gold display a little bit
	-- local font, size = GUI.infoHisContainer.children[5].label:GetFont()
	-- GUI.infoHisContainer.children[5].label:SetJustifyH("RIGHT")
	-- GUI.infoHisContainer.children[5].label:SetFont(font, 20)
	
	TSMAPI.GUI:UpdateTSMScrollingTableData("SID_HISTORY_ST", private.GetHistorySTItemData(GUI.hisFilters))
	private.GetHistoryInfoData(GUI.hisFilters)
end

function private.GetHistorySTItemData(lfilters)
	addon.Data:DebugPrintf("GetHistorySTItemData()")
	if GUI.view ~= GUI.VIEW_DATABASE or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end
	
	if (addon.hisCTag > 0) then
		addon.Data:DebugPrintf("tag = %s", addon.hisCTag)
		return private.GetSTItemData(addon.hisItems[addon.hisCTag], lfilters)
	else
		addon.Data:DebugPrintf("no tag active!")
		return private.GetSTItemData({}, lfilters)
	end
end

function private.GetHistoryInfoData(filters)
	addon.Data:DebugPrintf("GetHistoryInfoData()")
	if GUI.view ~= GUI.VIEW_DATABASE or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end

	if (addon.hisCTag > 0) then
		if (addon.hisInfo[addon.hisCTag].hisMVSource ~= addon.db.global.mvSource) then
			addon.Data:DebugPrintf(L["mvSource changed, update Data..."])
			addon.Data:UpdateDatabaseTag(addon.hisCTag)
		end
	end
	
	local stData = {}
	local mvFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local itStart = 0
	local itEnd = 0
	local isFiltered = false

	if (not filters.name and not filters.group and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		addon.Data:DebugPrintf("... with no filters")
		-- done
	else
		if (addon.hisCTag > 0) then
			addon.Data:DebugPrintf("... with filters")
			isFiltered = true
			itStart = 0
			itEnd = 0
			for itemString, data in pairs(addon.hisItems[addon.hisCTag]) do
				if #data.drops > 0 then
					itsSum = itsSum + 1
				end
				if #data.drops > 0 and not private:IsItemFiltered(itemString, filters) then
					itsFilter = itsFilter + 1
					for _, record in ipairs(data.drops) do
						itSum = itSum + record.quantity
						if not private:IsRecordFiltered(record, filters) then
							if (itStart == 0) then
								itStart = record.time
							end
							if (record.time < itStart) then
								itStart = record.time
							end
							if (record.time > itEnd ) then
								itEnd = record.time
							end
							itFilter = itFilter + record.quantity
							local lItemString = data.iString
							if (addon.db.global.useNotTheRealItemString) then
								lItemString = data.iBaseString
							end
							local marketValue = addon.Data:GetItemValue(lItemString) or 0
							mvFilter = mvFilter + (marketValue * record.quantity)
						end
					end
				end
			end
		end
		--addon.Data:DebugPrintf("  itsSum %s itsFilter %s itSum %s itFilter %s mvFilter %s nwFilter %s",
		--	itsSum, itsFilter, itSum, itFilter, mvFilter, nwFilter)
		--addon.Data:DebugPrintf("  sesMV %s sesGold %s sesNumItems %s sesNumNW %s",
		--	addon.db.realm.sesMV, addon.db.realm.sesGold, addon.db.realm.sesNumItems, addon.db.realm.sesNumNW)
	end
	
	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if (GUI.infoHisContainer) then
			-- Duration
			GUI.lblHisContainer.children[3]:SetText(L["Duration"])
			GUI.infoHisContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				SecondsToTime(itEnd - itStart) 
				.. FONT_COLOR_CODE_CLOSE)
			-- LIV / Gold
			GUI.lblHisContainer.children[5]:SetText(L["LIV"])
			GUI.infoHisContainer.children[5]:SetText(GUI:MoneyToString(mvFilter)) 
			-- GUI.infoHisContainer.children[5].label:SetAllPoints()
			-- Price Source
			GUI.infoHisContainer.children[7]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
		end
	else 
		if (GUI.infoHisContainer) then
			-- Duration
			GUI.lblHisContainer.children[3]:SetText(L["Duration"])
			if (addon.hisCTag > 0) then
				GUI.infoHisContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
					SecondsToTime(addon.hisInfo[addon.hisCTag].hisEnd - addon.hisInfo[addon.hisCTag].hisTime) 
					.. FONT_COLOR_CODE_CLOSE)
			end
			-- Looted Item Value / Gold
			GUI.lblHisContainer.children[5]:SetText(L["LIV"])
			if (addon.hisCTag > 0) then
				GUI.infoHisContainer.children[5]:SetText(GUI:MoneyToString(addon.hisInfo[addon.hisCTag].hisMV))
			end
			-- GUI.infoHisContainer.children[5].label:SetAllPoints()
			-- Price Source
			GUI.infoHisContainer.children[7]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
		end
	end
end

------------------------------------------------------------------------------
-- Challenge Tab
--
function GUI:LoadChallenge(container)
	addon.Data:DebugPrintf("GUI:LoadChallenge()")
	GUI.view = GUI.VIEW_CHALLENGE
	
	-- FIXME: this is a temporary layout, changes to this are coming
	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "Label",
					text = L["This page will help you to partitipate and organize Transmog Gear Challenge Runs."],
					relativeWidth = 1,
				},
			},
		},
	}
	
	TSMAPI.GUI:BuildOptions(container, page)
end

------------------------------------------------------------------------------
-- Inventory Tab
--
function GUI:LoadInventory(container)
	addon.Data:DebugPrintf("GUI:LoadInventory()")
	GUI.view = GUI.VIEW_INV
	
	-- Load History Data
	addon.invInfo = {}
	addon.invItems = addon.Data:LoadFromBags({})

	local zoneList = {}
	for _, data in pairs(addon.invItems) do
		for _, record in ipairs(data.drops) do
			zoneList[record.zone] = record.zone
		end
	end
	zoneList["all"] = L["All"]
	sort(zoneList, function(a, b) return (a.zone or 0) < (b.zone or 0) end)
	
	-- Filter: default filter is empty
	GUI.invFilters = {}

	-- compatibilty warning
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	local stHandlers = {
		OnEnter = function(_, data, self)
			if not data then return end
			if not data.itemString then
				addon.Data:DebugPrintf("Tooltip: data.itemString is nil!?")
			else
				local link = 
				TSMAPI.Item:ToItemLink(data.itemString)
				
				GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
				TSMAPI.Util:SafeTooltipLink(link)
			end
		end,
		OnLeave = function()
			GameTooltip:Hide()
		end,
	}

	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "InlineGroup",
					title = L["Inventory Summary"],
					layout = "Flow",
					children = {
						-- lblContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Label",
									text = L["LIV"],
									relativeWidth = 0.45,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.1,
								},
								{
									type = "Label",
									text = L["Price Source"], 
									relativeWidth = 0.45,
								},
							},
						},	
						-- infoContainer
						{
							type = "SimpleGroup",
							layout = "Flow",
							relativeWidth = 1,
							children = {
								{
									type = "Label",
									text = L["LIV"],
									relativeWidth = 0.45,
								},
								{
									type = "Label",
									text = "",
									relativeWidth = 0.1,
								},
								{
									type = "Label",
									text = "---",
									relativeWidth = 0.45,
								},
							},
						},	
					},
				},
				{
					type = "InlineGroup",
					title = L["Inventory Data"],
					layout = "TSMFillList",
					children = {
						{
							type = "SimpleGroup",
							layout = "Flow",
							children = {
								{
									type = "EditBox",
									label = L["Search"],
									relativeWidth = 0.20,
									onTextChanged = true,
									callback = function(_, _, value)
										value = value:trim()
										if value == "" then
											GUI.invFilters.name = nil
										else
											GUI.invFilters.name = TSMAPI.Util:StrEscape(value)
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_INVENTORY_ST", private.GetInventorySTItemData(GUI.invFilters))
										private.GetInventoryInfoData(GUI.invFilters)
									end,
								},
								{
									type = "GroupBox",
									label = L["Group"],
									relativeWidth = 0.20,
									callback = function(_, _, value)
										GUI.invFilters.group = value
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_INVENTORY_ST", private.GetInventorySTItemData(GUI.invFilters))
										private.GetInventoryInfoData(GUI.invFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Rarity"],
									relativeWidth = 0.20,
									list = GUI.rarityList,
									value = 0,
									callback = function(_, _, key)
										if key > 0 then
											GUI.invFilters.rarity = key
										else
											GUI.invFilters.rarity = nil
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_INVENTORY_ST", private.GetInventorySTItemData(GUI.invFilters))
										private.GetInventoryInfoData(GUI.invFilters)
									end,
								},
								{
									type = "Dropdown",
									label = L["Zone"],
									relativeWidth = 0.20,
									list = zoneList,
									value = "all",
									callback = function(_, _, value)
										if value == "all" then
											GUI.invFilters.zone = nil
										else
											GUI.invFilters.zone = value
										end
										TSMAPI.GUI:UpdateTSMScrollingTableData("SID_INVENTORY_ST", private.GetInventorySTItemData(GUI.invFilters))
										private.GetInventoryInfoData(GUI.invFilters)
									end,
								},
							},
						},
						{ 
							type = "ScrollingTable",
							tag = "SID_INVENTORY_ST",
							relativeWidth = 1,
							colInfo = {
								{name=L["Item"] .. flNotTheRealItemString, width=0.30, headAlign="LEFT"},
								{name=L["Item Value"], width=0.17, align="RIGHT", headAlign="LEFT"},
								{name=L["Qty"], width=0.05, align="RIGHT", headAlign="LEFT"},
								{name=L["Player"], width=0.12, headAlign="LEFT"},
								{name=L["Zone"], width=0.20, headAlign="LEFT"},
								{name=L["Time"], width=0.16, headAlign="LEFT"},
							},
							handlers = stHandlers,
							defaultSort = -6,
							selectionDisabled = true,
						},
					},
				},
			},
		},
	}

	local stInfo = page[1].children[2].children[2]
	if not addon.db.global.shPlayer then
		stInfo.colInfo = {
			{name=L["Item"] .. flNotTheRealItemString, width=0.40, headAlign="LEFT"},
			{name=L["Item Value"], width=0.16, align="RIGHT", headAlign="LEFT"},
			{name=L["Qty"], width=0.04, align="RIGHT", headAlign="LEFT"},
			{name="", width=0.01, headAlign="LEFT"},
			{name=L["Zone"], width=0.19, headAlign="LEFT"},
			{name=L["Time"], width=0.20, headAlign="LEFT"},
		}
	end
	
	TSMAPI.GUI:BuildOptions(container, page)
	
		-- yes, we can change label texts
	GUI.lblInvContainer = container.children[1].children[1].children[1]
	GUI.infoInvContainer = container.children[1].children[1].children[2]
	
	TSMAPI.GUI:UpdateTSMScrollingTableData("SID_INVENTORY_ST", private.GetInventorySTItemData(GUI.invFilters))
	private.GetInventoryInfoData(GUI.invFilters)
end

function private.GetInventorySTItemData(lfilters)
	addon.Data:DebugPrintf("GetInventorySTItemData()")
	if GUI.view ~= GUI.VIEW_INV or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end
	
	return private.GetSTItemData(addon.invItems, lfilters, true)
end

function private.GetInventoryInfoData(filters)
	addon.Data:DebugPrintf("GetInventoryInfoData()")
	if GUI.view ~= GUI.VIEW_INV or not GUI.isVisible then
		addon.Data:DebugPrintf("... nothing done.")
		return
	end

	if (addon.invInfo.invMVSource ~= addon.db.global.mvSource) then
		addon.Data:DebugPrintf(L["mvSource changed, update Data..."])
		addon.Data:UpdateInventory()
	end
	
	local stData = {}
	local mvFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local itStart = 0
	local itEnd = 0
	local isFiltered = false

	if (not filters.name and not filters.group and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		addon.Data:DebugPrintf("... with no filters")
		-- done
	else
		addon.Data:DebugPrintf("... with filters")
		isFiltered = true
		itStart = 0
		itEnd = 0
		for itemString, data in pairs(addon.invItems) do
			if #data.drops > 0 then
				itsSum = itsSum + 1
			end
			if #data.drops > 0 and not private:IsItemFiltered(itemString, filters) then
				itsFilter = itsFilter + 1
				for _, record in ipairs(data.drops) do
					itSum = itSum + record.quantity
					if not private:IsRecordFiltered(record, filters) then
						if (itStart == 0) then
							itStart = record.time
						end
						if (record.time < itStart) then
							itStart = record.time
						end
						if (record.time > itEnd ) then
							itEnd = record.time
						end
						itFilter = itFilter + record.quantity
						local lItemString = data.iString
						if (addon.db.global.useNotTheRealItemString) then
							lItemString = data.iBaseString
						end
						local marketValue = addon.Data:GetItemValue(lItemString) or 0
						mvFilter = mvFilter + (marketValue * record.quantity)
					end
				end
			end
		end
	end
	
	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if (GUI.infoInvContainer) then
			-- LIV / Gold
			GUI.infoInvContainer.children[1]:SetText(GUI:MoneyToString(mvFilter)) 
			-- Price Source
			GUI.infoInvContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
		end
	else 
		if (GUI.infoInvContainer) then
			-- Looted Item Value / Gold
			GUI.infoInvContainer.children[1]:SetText(GUI:MoneyToString(addon.invInfo.invMV))
			-- Price Source
			GUI.infoInvContainer.children[3]:SetText(ITEM_QUALITY_COLORS[6].hex .. 
				addon.db.global.mvSource 
				.. FONT_COLOR_CODE_CLOSE)
		end
	end
end

------------------------------------------------------------------------------
-- Tools Tab
--
function GUI:LoadTools(container)
	addon.Data:DebugPrintf("GUI:LoadTools()")
	GUI.view = GUI.VIEW_TOOLS
	
	-- FIXME: this is a temporary layout, changes to this are coming
	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "Label",
					text = L["This page will have some statistics, maintenance tools, tools to verify price sources etc."],
					relativeWidth = 1,
				},
			},
		},
	}
	
	TSMAPI.GUI:BuildOptions(container, page)
end

------------------------------------------------------------------------------
-- Information Tab
--
function GUI:LoadInfo(container)
	addon.Data:DebugPrintf("GUI:LoadInfo()")
	GUI.view = GUI.VIEW_INFO
	
	-- FIXME: this is a temporary layout, changes to this are coming
	local page = {
		{
			type = "SimpleGroup",
			layout = "TSMFillList",
			children = {
				{
					type = "Label",
					text = L["This page will show information about this addon and some usage information."],
					relativeWidth = 1,
				},
			},
		},
	}
	
	local info = page[1].children[1]

	info.text = 
		"SellableItemDrops Version " .. addon.version .. "\n\n" ..
		"SellableItemDrops remembers your looted items during an item farming session. " ..
		"The session item list shows name, time, drop location of every dropped item " ..
		"and it's market value based on a choosen TSM3 price source.\n" ..
		"\n" ..
		"The session data can be archived to a database and this database of previous " ..
		"farming runs can be searched for items and more.\n" ..
		"\n" ..
		"It supports competitive transmog challenge runs where you try to reach the " ..
		"highest accumulated market value of all dropped items during a given time period.\n" ..
		"\n" ..
		"The Session Tab and the History Tab are TSM3 modules and opens with '/tsm sid'.\n" ..
		"Addon options are configured through the TSM3 Module Options Tab 'SellableItemDrops'.\n" ..
		"\n" ..
		"SellableItemDrops is comparable to 'Loot Appraiser' and runs nicely together with " ..
		"'Loot Appraiser Challenge', so you can take screenshots of all your item drops " ..
		"to compare it with other players.\n" ..
		"\n" ..
		"SellableItemDrops requires TradeSkillMaster v3 and some " ..
		"working price sources. Not required but highly recommended is the use of the " ..
		"TSM Desktop App, so you can use the default price source 'DBGlobalMarketAvg'.\n\n" ..
		"SellableItemDrops is Work-In-Progress. Some additions incoming. Stay tuned ;-)\n\n" ..
		"You can find me as Caraxe on Curse.com/Curseforge.com for suggestions and comments."
	
	if (GetLocale() == "deDE") then
		info.text = 
		"Informationen zu SellableItemDrops Version " .. addon.version .. "\n\n" ..
		"SellableItemDrops merkt sich alle erbeuteten und verkaufbaren Gegenstände, " ..
		"die während einer Farmsitzung droppen. Es zeigt deren Namen, Fundort, Zeitpunkt " ..
		"und Marktwert, der durch eine einstellbare Trade Skill Master 3-Preisquelle ermittelt wird.\n\n" ..
		"Alle Gegenstände werden in einer " ..
		"sortierbaren und durchsuchbaren Liste angezeigt. Über eine integrierte " ..
		"Chronik können auch die Ergebnisse vergangener Sitzungen angezeigt werden.\n" ..
		"\n" ..
		"Zusätzlich werden 'Transmog Challenge Runs' unterstützt, " ..
		"bei denen im Wettstreit mit anderen versucht wird, den höchsten Marktwert " ..
		"aller erbeuteten Gegenstände während einer bestimmten Zeitspanne zu erreichen.\n" ..
		"\n" ..
		"SellableItemDrops ist vergleichbar mit 'Loot Appraiser'. Es kann parallel " ..
		"zu 'Loot Appraiser Challenge' verwendet werden und sorgt dann im Hintergrund " ..
		"für eine sortierbare Liste aller gedroppten Gegenstände um lohnende Farmorte zu identifizieren oder um gefundene Gegenstände " ..
		"für sich oder andere zu dokumentieren.\n\n" ..
		"SellableItemDrops ist ein TSM3-Modul und benötigt daher TradeSkillMaster (Version 3) mit " ..
		"funktionierenden Preisquellen. Optimal ist es, wenn die " ..
		"TSM Desktop App installiert wurde und damit die Preisquelle 'DBGlobalMarketAvg' zur Verfügung steht.\n\n" ..
		"Die Entwicklung von SellableItemDrops ist noch nicht abgeschlossen. Bereits geplant ist: automatisches Starten/Stoppen bei " ..
		"Challenges, CSV- und HTML-Export von Daten, Statistik- und Auswertefunktionen basierend auf TSM Auktionsdaten, Zeitleiste für Instanz-Lockouts.\n" ..
		"\n" ..
		"Für Wünsche und Fragen bin ich als Caraxe auf Curse.com/Curseforge.com zu erreichen."	
	end

	TSMAPI.GUI:BuildOptions(container, page)
end

------------------------------------------------------------------------------
-- Utility Functions
--
function GUI:TimeToString(t)
	if addon.db.global.timeFormat == "ago" then
		local lTime = time() - t
		if (lTime == 0) then
			return L["now"]
		else
			return format(L["%s ago"],  SecondsToTime(lTime) or "?")
		end
	elseif (addon.db.global.timeFormat) then
		return date(addon.db.global.timeFormat or addon.timeFormat, t)
	end
end

function GUI:MoneyToString(money, ...)
	return TSMAPI:MoneyToString(money, "OPT_PAD", ...)
end

function private:IsItemFiltered(itemString, filters)
	local lItemString = itemString
	if (addon.db.global.useNotTheRealItemString) then
		lItemString = TSMAPI.Item:ToBaseItemString(lItemString)
	end
	local name, _, rarity = TSMAPI.Item:GetInfo(lItemString)
	rarity = rarity or 0
	if not name then return true end

	if filters.name and not strfind(strlower(name), strlower(filters.name)) then
		return true
	end

	if filters.rarity and rarity < filters.rarity then
		return true
	end

	if filters.group then
		local groupPath = TSMAPI.Groups:GetPath(lItemString) or TSMAPI.Groups:GetPath(TSMAPI.Item:ToBaseItemString(lItemString))
		if not groupPath or not strfind(groupPath, "^" .. TSMAPI.Util:StrEscape(filters.group)) then
			return true
		end
	end
end

function private:IsRecordFiltered(record, filters)
	local SECONDS_PER_DAY = 24 * 60 * 60
	if filters.player and record.player ~= filters.player then
		return true
	end
	if filters.zone and record.zone ~= filters.zone then
		return true
	end
	if filters.time and floor(record.time/SECONDS_PER_DAY) < (floor(time()/SECONDS_PER_DAY) - filters.time) then
		return true
	end
end

-- EOF
